@@ -1,6 +1,8 @@
 package String::Approx;
 
-$VERSION = '3.24';
+require v5.8.0;
+
+$VERSION = '3.27';
 
 use strict;
 local $^W = 1;
@@ -122,7 +124,8 @@ sub _parse_param {
             } elsif (s/^\?//) {
                 $param{'?'} = 1;
             } else {
-                die "unknown parameter: '$_'\n";
+                warn "unknown parameter: '$_'\n";
+                return;
             }
         }
     }
@@ -261,7 +264,9 @@ sub amatch {
         }
     } 
     return $a->match($_) if defined $_;
-    die "amatch: \$_ is undefined: what are you matching?\n";
+
+    warn "amatch: \$_ is undefined: what are you matching?\n";
+    return;
 }
 
 sub _find_substitute {
@@ -344,7 +349,8 @@ sub asubstitute {
 	}
 	return $_ = $n[0];
     } else {
-	die "asubstitute: \$_ is undefined: what are you substituting?\n";
+	warn "asubstitute: \$_ is undefined: what are you substituting?\n";
+        return;
     }
 }
 
@@ -364,7 +370,9 @@ sub aindex {
 	}
     }
     return $a->index($_) if defined $_;
-    die "aindex: \$_ is undefined: what are you indexing?\n";
+
+    warn "aindex: \$_ is undefined: what are you indexing?\n";
+    return;
 }
 
 sub aslice {
@@ -379,7 +387,9 @@ sub aslice {
 	return map { [ $a->slice($_) ] } @_;
     }
     return $a->slice($_) if defined $_;
-    die "aslice: \$_ is undefined: what are you slicing?\n";
+
+    warn "aslice: \$_ is undefined: what are you slicing?\n";
+    return;
 }
 
 sub _adist {
@@ -394,6 +404,12 @@ sub _adist {
 sub adist {
     my $a0 = shift;
     my $a1 = shift;
+    if (length($a0) == 0) {
+      return length($a1);
+    }
+    if (length($a1) == 0) {
+      return length($a0);
+    }
     my @m = ref $_[0] eq 'ARRAY' ? @{shift()} : ();
     if (ref $a0 eq 'ARRAY') {
 	if (ref $a1 eq 'ARRAY') {
@@ -475,11 +491,14 @@ sub arindex {
 	my $aindex = $a->index(scalar reverse());
 	return $aindex == -1 ? $aindex : (length($_) - $aindex - $l);
     }
-    die "arindex: \$_ is undefined: what are you indexing?\n";
+
+    warn "arindex: \$_ is undefined: what are you indexing?\n";
+    return;
 }
 
 1;
 __END__
+=pod
 
 =head1 NAME
 
@@ -502,15 +521,21 @@ With this you can emulate errors: typing errorrs, speling errors,
 closely related vocabularies (colour color), genetic mutations (GAG
 ACT), abbreviations (McScot, MacScot).
 
-NOTE: String::Approx has been designed to work with B<strings>, not
-with B<text>.  In other words, when you want to compare things like
-text or source code, consisting of B<words> or B<tokens> and
-B<phrases> and B<sentences>, or B<expressions> and B<statements>,
-you should probably use some other tool than String::Approx, like for
-example the standard UNIX diff(1) tool, or the Algorithm::Diff module
-from CPAN, or if you just want the Levenshtein edit distance
-(explained below), the Text::Levenshtein module from CPAN.  See also
-Text::WagnerFischer and Text::PhraseDistance.
+NOTE: String::Approx suits the task of B<string matching>, not 
+B<string comparison>, and it works for B<strings>, not for B<text>.
+
+If you want to compare strings for similarity, you probably just want
+the Levenshtein edit distance (explained below), the Text::Levenshtein
+and Text::LevenshteinXS modules in CPAN.  See also Text::WagnerFischer
+and Text::PhraseDistance.  (There are functions for this in String::Approx,
+e.g. adist(), but their results sometimes differ from the bare Levenshtein
+et al.)
+
+If you want to compare things like text or source code, consisting of
+B<words> or B<tokens> and B<phrases> and B<sentences>, or
+B<expressions> and B<statements>, you should probably use some other
+tool than String::Approx, like for example the standard UNIX diff(1)
+tool, or the Algorithm::Diff module from CPAN.
 
 The measure of B<approximateness> is the I<Levenshtein edit distance>.
 It is the total number of "edits": insertions,
@@ -532,6 +557,12 @@ transform I<"lead"> into I<"gold">, you need three edits:
 
 The edit distance of "lead" and "gold" is therefore three, or 75%.
 
+B<String::Approx> uses the Levenshtein edit distance as its measure, but
+String::Approx is not well-suited for comparing strings of different
+length, in other words, if you want a "fuzzy eq", see above.
+String::Approx is more like regular expressions or index(), it finds
+substrings that are close matches.>
+
 =head1 MATCH
 
 	use String::Approx 'amatch';
@@ -610,6 +641,9 @@ means I<ignore case>, I<allow every fourth character to be "an edit">,
 but allow I<no substitutions>.  (See L<NOTES> about disallowing
 substitutions or insertions.)
 
+NOTE: setting C<I0 D0 S0> is not equivalent to using index().
+If you want to use index(), use index().
+
 =head1 SUBSTITUTE
 
 	use String::Approx 'asubstitute';
@@ -646,11 +680,18 @@ matches approximately.  In list context and if C<@inputs> are used,
 returns a list of indices, one index for each input element.
 If there's no approximate match, C<-1> is returned as the index.
 
+NOTE: if there is character repetition (e.g. "aa") either in
+the pattern or in the text, the returned index might start 
+"too early".  This is consistent with the goal of the module
+of matching "as early as possible", just like regular expressions
+(that there might be a "less approximate" match starting later is
+of somewhat irrelevant).
+
 There's also backwards-scanning C<arindex()>.
 
 =head1 SLICE
 
-	use String::Approx 'aindex';
+	use String::Approx 'aslice';
 
 	($index, $size)   = aslice("pattern")
 	([$i0, $s0], ...) = aslice("pattern", @inputs)
@@ -661,9 +702,10 @@ Like C<aindex()> but returns also the size (length) of the match.
 If the match fails, returns an empty list (when matching against C<$_>)
 or an empty anonymous list corresponding to the particular input.
 
-Note that the size of the match will very probably be something you
-did not expect (such as longer than the pattern, or a negative
-number).  This may or may not be fixed in future releases.
+NOTE: size of the match will very probably be something you did not
+expect (such as longer than the pattern, or a negative number).  This
+may or may not be fixed in future releases. Also the beginning of the
+match may vary from the expected as with aindex(), see above.
 
 If the modifier
 
@@ -856,11 +898,12 @@ Aldo Calpini, David Curiel, Teun van den Dool, Alberto Fontaneda,
 Rob Fugina, Dmitrij Frishman, Lars Gregersen, Kevin Greiner,
 B. Elijah Griffin, Mike Hanafey, Mitch Helle, Ricky Houghton,
 'idallen', Helmut Jarausch, Damian Keefe, Ben Kennedy, Craig Kelley,
-Franz Kirsch, Dag Kristian, Mark Land, J. D. Laub, Tim Maher,
-Juha Muilu, Sergey Novoselov, Andy Oram, Ji Y Park, Eric Promislow,
-Nikolaus Rath, Stefan Ram, Dag Kristian Rognlien, Stewart Russell,
-Slaven Rezic, Chris Rosin, Pasha Sadri, Ilya Sandler, Bob J.A. Schijvenaars,
-Ross Smith, Frank Tobin, Greg Ward, Rich Williams, Rick Wise.
+Franz Kirsch, Dag Kristian, Mark Land, J. D. Laub, John P. Linderman,
+Tim Maher, Juha Muilu, Sergey Novoselov, Andy Oram, Ji Y Park,
+Eric Promislow, Nikolaus Rath, Stefan Ram, Slaven Rezic,
+Dag Kristian Rognlien, Stewart Russell, Slaven Rezic, Chris Rosin,
+Pasha Sadri, Ilya Sandler, Bob J.A. Schijvenaars, Ross Smith,
+Frank Tobin, Greg Ward, Rich Williams, Rick Wise.
 
 The matching algorithm was developed by Udi Manber, Sun Wu, and Burra
 Gopal in the Department of Computer Science, University of Arizona.
@@ -871,10 +914,12 @@ Jarkko Hietaniemi <jhi@iki.fi>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2001-2005 by Jarkko Hietaniemi
+Copyright 2001-2013 by Jarkko Hietaniemi
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+under either the terms of the Artistic License 2.0, or the GNU Library
+General Public License, Version 2.  See the files Artistic and LGPL
+for more details.
 
 Furthermore: no warranties or obligations of any kind are given, and
 the separate file F<COPYRIGHT> must be included intact in all copies
